/*
Ver. 1.01 no bitshift - 2019-09-14

Based on : Ver. 1.01 - 2015-06-17

This sketch emulates Thrustmaster T500RS stock GT wheel
allowing to connect arduino to T500RS wheelbase (may work for TX RW/T300 as well)
and emulate button presses. GT wheel has 13 buttons - this sketch emulates all.

!!! There are 2 flavours of arduino boards - powered by +5 volts or +3.3 volts !!!
!!! If you have +5V Arduino version - disconnect it from USB before connecting to a wheelbase !!!
!!! Just power your arduino from TX RW wheelbase Black pin - it supplies +3.3V voltage (should be enough) !!!
!!! don't connect USB +5V wire to TX RW wheelbase Black +3.3V - you'll get a collision !!!

Thrustmaster Wheelbase cable pinout (PS/2 connector, also known as mini-Din 6pin female):
1 - Green    - not used
2 - Blue     - GND (ground)
3 - White    - MISO (master in, slave out - to read the data from the wheel)
4 - Orange   - SS (slave select, or PL - parallel load, set it to 0 when you want to read the data)
5 - Red      - CLK (clock impulses)
6 - Black    - +VCC (+3.3 volts! if you have Arduino 5V version - use external +3.3V power or use the wheelbase power!!!)

Arduino UNO pins -> TX RW wheelbase cable pins

Arduino GND                 -> Wheelbase Blue wire (2)
Arduino pin 12              -> Wheelbase White wire (3) (data from wheel to base)
Arduino pin 10 + pin 2 (SS) -> Wheelbase Orange wire pin (4) (yes, on Arduino it's wired to two pins. 10 - SS, 2 - INT0)
Arduino pin 13 (SCK)        -> Wheelbase Red wire (5)
Arduino +5V                 -> Wheelbase Black wire (6) (it gives 3.3V, but must be connected into +5V socket on arduino uno side)

Button mappings (we send 8 bytes to wheelbase, only 2 first bytes matter, only 13 bits are used for buttons)

Byte 1
  0 - not used
  0 - n/u
  0 - n/u
  1 - R2 Button #9           -> Arduino digital pin D7 (portD bit7)
  1 - Triangle Button #3     -> pin D6 (portD bit6)
  1 - Circle Button #5       -> pin D5 (portD bit5)
  1 - Square Button #4       -> pin D4 (portD bit4)
  1 – Cross Button #6        -> pin D3 (portD bit3)
  
Byte 2
  1 – Select Button #7       -> pin D9 (portB bit1)
  1 – L2 Button #10          -> pin D8 (portB bit0)
  1 – PS Button #13          -> pin A5 (portC bit5)
  1 – Start Button #8        -> pin A4 (portC bit4)
  1 – D-Pad Down             -> pin A3 (portC bit3)
  1 - D-Pad Right            -> pin A2 (portC bit2)
  1 - D-Pad Left             -> pin A1 (portC bit1)
  1 - D-Pad Up               -> pin A0 (portC bit0)
  
Ah, each push button has 2 pins - you connect one to Arduino, another to the Ground :)
Pressed button gives you "0" (grounded signal), released = "1" (pulled-up to +VCC)
  
Free free to modify and use for your needs.
This sketch and the documentation above provided "AS IS" under the BSD New license.
http://opensource.org/licenses/BSD-3-Clause
(c) Taras Ivaniukovich (blog@rr-m.org) July 2015

Thanks to:
Seth C. for T500 GT rim data structure, button mapping and beta-testing
Oleg P. for testing with T500RS on fresh 2019 firmware
*/

//#include "pins_arduino.h" // not necessary in 1.6.3?

byte wheelState [8];
volatile byte pos;

void setup (void) {
  DDRB  |= B00001011; // digital pins 8,9,11 used as inputs with a pull-up to +VCC
  PORTB |= B00001011;
  
  DDRC |= B00111111; // pins 14-19 (A0 - A5) also used as digital inputs
  PORTC |= B00111111; // pulled-up to +VCC via internal 100k resistors
  
  DDRD  |= B11111011; // digital pins 0,1,3,4,5,6,7 used as inputs
  PORTD |= B11111011; // pulled-up to +VCC via internal 100k resistors
  
  wheelState[0] = B00011111; // T500 GT wheel first data byte - 5 buttons
  wheelState[1] = B11111111; // second data byte - 8 buttons
  wheelState[2] = B11111111; // this and below - not used, but wheelbase reads all 8 bytes...
  wheelState[3] = B11111111; 
  wheelState[4] = B11111111;
  wheelState[5] = B11111111;
  wheelState[6] = B11111111;
  wheelState[7] = B11111111;

//  Serial.begin(9600);    // Arduino debug console - occupies pins RX (0) and TX (1) on Uno
  pinMode(MISO, OUTPUT); // arduino is a slave device
  SPCR |= _BV(SPE);      // turn on SPI in slave mode
  SPCR |= _BV(SPIE);     // turn on interrupts
  
  // interrupt for SS rising edge. Arduino Uno Pin10 must be connected to Pin2!!!
  attachInterrupt (0, ss_rising, RISING);
}

// Interrupt0 (external, pin 2) - prepare to start the transfer
void ss_rising () {
  SPDR = wheelState[0]; // load first byte into SPI data register
  pos = 1;
}

// SPI interrupt routine
ISR (SPI_STC_vect) {
  SPDR = wheelState[pos++]; // load the next byte to SPI output register and return.
}

void loop() {
  // scan the button presses and save that to wheelState array. Data transfer to wheelbase is interrupt-driven above.
  
  /* This doesn't work with T500RS base - we need to shift 1 bit left before data transfer */
  /* Update Sep 2019 - this works with T500RS on firmware 47.00 (freshest mid 2019) */
  wheelState[0] = PIND >> 3;                                      // T500 GT byte1 - take bits 3-7 from PORTD
  wheelState[1] = (PINC & B00111111) | ((PINB & B00000011) << 6); // T500 GT byte2 - take bits 0-5 from PORTC + the rest from PORTB Bxxxxxx11

  // everything is shifted 1 bit left - compared to this sketch description (works for on old T500 firmware, fails in Sep 2019)
  /* wheelState[0] = ((PIND & B11111000) >> 2 ) | ((PINB & B00000010) >> 1);           // T500 GT byte1 - take bits 3-7 from PORTD + Bit 1 from B
  wheelState[1] = ((PINC & B00111111) << 1) | ((PINB & B00000001) << 7) | B0000001; // T500 GT byte2 - take bits 0-5 from PORTC + Bit 0 from B  */
}
